##############################################################################
#
#	CSCcrypt::GPG	-	This module provides the interfacing
#		required to get CSCmail to interface to GPG.  This module
#		is defined to be as generic as possible and I encourage
#		any use for any other project to feed back any bug fixes
#		or modifications back to us so we can all benefit.
#
#	Author		-	Redvers Davies <red@madhouse.org.uk>
#
#	Copyright 2000 Aztek Networking Ltd
#	Licenced under the GNU Public Licence (GPL).
#
#	$Id: GPG.pm,v 1.9 2000/05/09 09:04:51 noidd Exp $
#
##############################################################################

use strict;
package CSCcrypt::GPG;

use IPC::Open3;

## Method new: Used to initialise the GPG support.

sub new {
	my $class = shift;
	my $self = {};
	my $hashref = shift;

	wr_debug("new $hashref");

	foreach my $key (keys %$hashref) {
		$self->{$key} = $hashref->{$key};
	}

	bless $self;
	return($self);
}

## Method to see if keyid specified is in the local keyring

sub is_key_local {
	my $self = shift;
	my $pattern = shift;

	wr_debug("KeyID to check is $pattern");
	wr_debug("Using GPG binary at: ".$self->{gpgpath});

	wr_debug("$self->{gpgpath} --fingerprint --with-colons $pattern 2> /dev/null");
	my @gpg_raw = `$self->{gpgpath} --fingerprint --with-colons $pattern 2> /dev/null`;
	chomp(@gpg_raw);

	## Return hash in following manner:
	## $rethash->{keyid}{bits,created,string,fingerprint...}

	my $counter = 0;
	my $keyid;		## Has to be sticky
	my $returnhash = {};	## Define it properly

	while ($counter < $#gpg_raw) {
		wr_debug("Line: $gpg_raw[$counter]");
		if ($gpg_raw[$counter] =~ /^pub/) {
			my @public_array = split(/:/, $gpg_raw[$counter]);
			my $longid = $public_array[4];
			wr_debug("Long ID: $longid");
			$longid =~ /(........)$/;
			$keyid = $1;

			$returnhash->{"0x$keyid"}{trust} = $public_array[1];
			$returnhash->{"0x$keyid"}{length} = $public_array[2];
			$returnhash->{"0x$keyid"}{algorithm} = $public_array[3];
			$returnhash->{"0x$keyid"}{fullkeyid} = $public_array[4];
			$returnhash->{"0x$keyid"}{created} = $public_array[5];
			$returnhash->{"0x$keyid"}{expires} = $public_array[6];
			$returnhash->{"0x$keyid"}{localid} = $public_array[7];
			$returnhash->{"0x$keyid"}{ownertrust} = $public_array[8];
			$returnhash->{"0x$keyid"}{string} = $public_array[9];

			wr_debug("Keyid: 0x$keyid");
			wr_debug("Length: $returnhash->{\"0x$keyid\"}{length}");
			wr_debug("Algorithm: $returnhash->{\"0x$keyid\"}{algorithm}");
			wr_debug("FullkeyID: $returnhash->{\"0x$keyid\"}{fullkeyid}");
			wr_debug("Created: $returnhash->{\"0x$keyid\"}{created}");
			wr_debug("Expires: $returnhash->{\"0x$keyid\"}{expires}");
			wr_debug("LocalID: $returnhash->{\"0x$keyid\"}{localid}");
			wr_debug("Ownertrust: $returnhash->{\"0x$keyid\"}{ownertrust}");
			wr_debug("String: $returnhash->{\"0x$keyid\"}{string}");

			$counter++;
			next;
		}

		if ($gpg_raw[$counter] =~ /^fpr/) {
			my @fingerprint_array = split(/:/, $gpg_raw[$counter]);
			wr_debug("0x$keyid => $fingerprint_array[9]");
			$returnhash->{"0x$keyid"}{fingerprint} = $fingerprint_array[9];
			$counter++;
			next;
		}

	$counter++;
		
	}

	return(1, $returnhash);

}

sub encrypt {
	my $self = shift;
	my $keyarray = shift;
	my $textarray = shift;
	my $encrypt = shift;
	my $sign = shift;
	my $passphrase = shift;
	my @flags;
	my $pid;

	wr_debug("Called encrypt function");
	if ($encrypt) {
		wr_debug("I will encrypt");
	}
	if ($sign) {
		wr_debug("I will also attempt to sign this email");
	}

	foreach my $user (@$keyarray) {
		push(@flags, "-r $user ");
	}

	if (($encrypt) && ($sign)) {
		$pid = open3(\*WTRFH, \*RDRFH, \*ERRFH, "$self->{gpgpath} --status-fd 2 --batch --passphrase-fd 0 --no-tty -sea @flags");
	} elsif (($encrypt) && (!($sign))) {
		$pid = open3(\*WTRFH, \*RDRFH, \*ERRFH, "$self->{gpgpath} --status-fd 2 --batch --passphrase-fd 0 --no-tty -ea @flags");
	} elsif ((!($encrypt)) && ($sign)) {
		$pid = open3(\*WTRFH, \*RDRFH, \*ERRFH, "$self->{gpgpath} --status-fd 2 --batch --passphrase-fd 0 --no-tty --clearsign");
	} else {
		wr_debug("Why were we called in the first place?");
		return 1;
	}
		

	wr_debug($self->{gpgpath}." -ea @flags");
	wr_debug("pid: $pid");

	if ($sign) {
		print(WTRFH "$passphrase\n");
	}

	foreach my $line_in (@$textarray) {
		print(WTRFH "$line_in");
	}

	close(WTRFH);

	my @stdout = <RDRFH>;
	wr_debug("STDOUT: @stdout");
	close(RDRFH);
	
	my @sterr = <ERRFH>;
	@sterr = grep(/\[GNUPG:\]/, @sterr);
	wr_debug("STDERR: @sterr");
	close(ERRFH);

	if ($stdout[0] !~ /--BEGIN/) {
		return([], \@sterr);
	}

	return(\@stdout, \@sterr);

}

sub decrypt {
	my $self = shift;
	my $messageref = shift;
	my $passphrase = shift;

	wr_debug("decrypt\($messageref, passphrase\) has been called");

	my $pid = open3(\*WTRFH, \*RDRFH, \*ERRFH, "$self->{gpgpath} --status-fd 2 --batch --passphrase-fd 0 --no-tty");


}


sub search_server {
	my $self = shift;
	my $server = shift;
	my $string = shift;

	wr_debug("search_server\($server, $string\)");
	my $teststring = "http://wwwkeys.pgp.net:11371/pks/lookup?op=vindex&search=redvers&fingerprint=on";
	
	my $serverobj = new IO::Socket::INET(PeerAddr => "$server", PeerPort => "11371", Proto => "tcp");
	print($serverobj "GET http://wwwkeys.pgp.net:11371/pks/lookup?op=vindex&search=redvers&fingerprint=on");
	print($serverobj "\n\n");

	my @raw_server = <$serverobj>;

	close($serverobj);

	wr_debug("@raw_server");





}

























#objectref->import_key_from_server
#objectref->send_key_to_server
#objectref->localsign
#objectref->realsign

sub wr_debug {
	print("@_\n");
}

1;
